@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -102,7 +103,9 @@ gkrellm_clear_chart_pixmap(Chart *cp)
 void
 gkrellm_clean_bg_src_pixmap(Chart *cp)
 	{
-	if (cp)
+	if (!cp)
+		return;
+	if (!draw_rootpixmap_onto_transparent_chart(cp))
 		gdk_draw_pixmap(cp->bg_src_pixmap, GK.draw1_GC,
 				cp->bg_clean_pixmap, 0, 0, 0, 0, cp->w, cp->h);
 	}
