@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -227,20 +228,32 @@ assign_textcolor(Style *style, gchar *arg, gint AorB)
 	g_free(values);
 	}
 
+void
+gkrellm_set_krell_expand(Style *style, gchar *value)
+	{
+	gint	expand	= KRELL_EXPAND_NONE;
+
+	if (!style)
+		return;
+	if (value)
+		{
+		if (!strcmp(value, "left"))
+			expand = KRELL_EXPAND_LEFT;
+		else if (!strcmp(value, "right"))
+			expand = KRELL_EXPAND_RIGHT;
+		else if (!strcmp(value, "bar-mode"))
+			expand = KRELL_EXPAND_BAR_MODE;
+		}
+	style->krell_expand = expand;
+	}
+
 static void
 assign_style_entry(Style *style, gchar *value, gint entry_flag)
 	{
 	if (entry_flag == SET_KRELL_YOFF)
 		style->krell_yoff = atoi(value);
 	else if (entry_flag == SET_KRELL_EXPAND)
-		{
-		if (strcmp(value, "left") == 0)
-			style->krell_expand = KRELL_EXPAND_LEFT;
-		else if (strcmp(value, "right") == 0)
-			style->krell_expand = KRELL_EXPAND_RIGHT;
-		else
-			style->krell_expand = KRELL_EXPAND_NONE;
-		}
+		gkrellm_set_krell_expand(style, value);
 	else if (entry_flag == SET_KRELL_X_HOT)
 		style->krell_x_hot = atoi(value);
 	else if (entry_flag == SET_KRELL_EMA_PERIOD)
@@ -968,7 +981,8 @@ static gchar
 			*button_panel_border,
 			*button_meter_border,
             *bg_slider_panel_border,
-            *bg_slider_meter_border;
+            *bg_slider_meter_border,
+			*krell_slider_expand;
 
 gint		krell_slider_depth,
 			krell_slider_x_hot;
@@ -1033,6 +1047,7 @@ static struct	_config
 
 	{"krell_slider_depth",	&krell_slider_depth,	NULL,			1  },
 	{"krell_slider_x_hot",	&krell_slider_x_hot,	NULL,			-1  },
+	{"krell_slider_expand",	NULL, 		&krell_slider_expand,		-1  },
 	};
 
 
@@ -1059,6 +1074,7 @@ cleanup_gkrellmrc(void)
 
 	GK.krell_slider_style->krell_x_hot  = krell_slider_x_hot;
 	GK.krell_slider_style->krell_depth  = krell_slider_depth;
+	gkrellm_set_krell_expand(GK.krell_slider_style, krell_slider_expand);
 	}
 
 static GList	*gkrellmrc_border_list,
@@ -1130,9 +1146,10 @@ gkrellm_get_gkrellmrc_image_border(gchar *image_name, GdkImlibImage *image,
 gboolean
 gkrellm_get_gkrellmrc_integer(gchar *int_name, gint *result)
 	{
-	GList	*list;
-	gchar	name[64], string[64];
-	gchar	*s, *r;
+	GList		*list;
+	gchar		name[64], string[64];
+	gchar		*s, *r;
+	gboolean	found = FALSE;
 
 	if (!int_name || !result)
 		return FALSE;
@@ -1142,14 +1159,10 @@ gkrellm_get_gkrellmrc_integer(gchar *int_name, gint *result)
 		if ((r = strchr(s, '=')) != NULL)
 			*r = ' ';
 		sscanf(s, "%64s %64s", name, string);
-		if (!strcmp(name, int_name))
-			{
-			if (sscanf(string, "%d", result) == 1)
-				return TRUE;
-			break;
-			}
+		if (!strcmp(name, int_name) && sscanf(string, "%d", result) == 1)
+			found = TRUE;
 		}
-	return FALSE;
+	return found;
 	}
 
 gchar *
@@ -1462,6 +1475,7 @@ init_theme(void)
 
 	krell_slider_x_hot = -1;
 	krell_slider_depth = 6;
+	gkrellm_dup_string(&krell_slider_expand, "none");
 
 	free_glist_and_data(&gkrellmrc_border_list);
 	free_glist_and_data(&gkrellmrc_integer_list);
@@ -1575,7 +1589,7 @@ static gchar	*default_theme[]	=
 	"spacer_top_height volume 3",
 	"spacer_bottom_height volume 3",
 	"StyleMeter volume.label_position = 0",
-	"StyleMeter volume.border = 26,3,1,1",
+	"StyleMeter volume.border = 26,3,7,1",
 	"StyleMeter volume.top_margin = 1",
 	"StyleMeter volume.bottom_margin = 0",
 	};
