@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -107,7 +108,7 @@
 */
 #define	GKRELLM_VERSION_MAJOR	1
 #define	GKRELLM_VERSION_MINOR	2
-#define	GKRELLM_VERSION_REV		4
+#define	GKRELLM_VERSION_REV		8
 /* Backward compatibility */
 #define	VERSION_MAJOR			GKRELLM_VERSION_MAJOR
 #define	VERSION_MINOR			GKRELLM_VERSION_MINOR
@@ -349,9 +350,10 @@ typedef struct
   |  wall full of power monitoring meters the Krell had in Forbidden Planet.
   */
   /* Krell.expand values */
-#define	KRELL_EXPAND_NONE	0
-#define	KRELL_EXPAND_LEFT	1
-#define	KRELL_EXPAND_RIGHT	2
+#define	KRELL_EXPAND_NONE		0
+#define	KRELL_EXPAND_LEFT		1
+#define	KRELL_EXPAND_RIGHT		2
+#define	KRELL_EXPAND_BAR_MODE	3
 
 typedef struct
 	{
@@ -384,6 +386,7 @@ typedef struct
 	gint		period;		/* of ema */
 	gint		full_scale_expand;
 	gboolean	monotonic;
+	gboolean	bar_mode;
 
 	Draw_rec	old_draw;	/* Last draw for restoring background.	*/
 	Draw_rec	draw;		/* Parameters to currently drawn krell	*/
@@ -593,20 +596,22 @@ typedef struct
 
 typedef struct
 	{
-	Panel	*p;
-	Decal	*d;
-	void	(*cb_button_event)();
-	gint	(*cb_in_button)();
-	void	*data;
-	gint	cur_index;
-	gint	pressed_index;
-	gint	saved_index;
-	gint	sensitive;
-	gint	type;
-	gint	id_press,
-			id_release,
-			id_enter,
-			id_leave;
+	Panel		*p;
+	Decal		*d;
+	void		(*cb_button_click)();
+	gint		(*cb_in_button)();
+	gpointer	data;
+	gint		cur_index;
+	gint		pressed_index;
+	gint		saved_index;
+	gint		sensitive;
+	gint		type;
+	gint		id_press,
+				id_release,
+				id_enter,
+				id_leave;
+	void		(*cb_button_right_click)();
+	gpointer	right_data;
 	}
 	DecalButton;
 
