@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -282,7 +283,7 @@ gkrellm_launcher_table_new(GtkWidget *vbox, gint n_launchers)
 	{
 	GtkWidget	*table;
 
-	table = gtk_table_new(2 * n_launchers, 3, TRUE /*homogeneous*/);
+	table = gtk_table_new(2 * n_launchers, 2, FALSE /*non-homogeneous*/);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 2);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 2);
 	return table;
@@ -301,7 +302,8 @@ gkrellm_config_launcher(GtkWidget *table, gint n, GtkWidget **launch_entry,
 	row = (tooltip_entry ? 2 : 1) * n;
 	hbox = gtk_hbox_new(FALSE, 0);
     /* Attach left right top bottom */
-    gtk_table_attach_defaults(GTK_TABLE(table), hbox, 0, 1, row, row + 1);
+    gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, row, row + 1,
+				GTK_SHRINK, GTK_SHRINK, 0, 0);
 
 	snprintf(buf, sizeof(buf), _("%s command:"), desc);
     label = gtk_label_new(buf);
@@ -310,15 +312,16 @@ gkrellm_config_launcher(GtkWidget *table, gint n, GtkWidget **launch_entry,
 
     *launch_entry = gtk_entry_new_with_max_length(255);
     gtk_table_attach_defaults(GTK_TABLE(table), *launch_entry,
-                1, 3, row, row + 1);
+                1, 2, row, row + 1);
 	gtk_entry_set_text(GTK_ENTRY(*launch_entry),
 					(launch && launch->command) ? launch->command : "");
 
 	if (tooltip_entry)
 		{
 		hbox = gtk_hbox_new(FALSE, 0);
-	    gtk_table_attach_defaults(GTK_TABLE(table), hbox,
-					0, 1, row + 1, row + 2);
+	    gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, row + 1, row + 2,
+					GTK_SHRINK, GTK_SHRINK, 0, 0);
+
 		snprintf(buf, sizeof(buf), _("%s comment:"), desc);
 	    label = gtk_label_new(buf);
 		gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -375,11 +378,12 @@ gkrellm_check_button(GtkWidget *box, GtkWidget **button, gboolean active,
 	{
 	GtkWidget	*b;
 
-	if (!string || !box)
+	if (!string)
 		return;
 	b = gtk_check_button_new_with_label(string);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), active);
-	gtk_box_pack_start(GTK_BOX(box), b, expand, FALSE, pad);
+	if (box)
+		gtk_box_pack_start(GTK_BOX(box), b, expand, FALSE, pad);
 	if (button)
 		*button = b;
 	}
@@ -391,11 +395,12 @@ gkrellm_check_button_connected(GtkWidget *box, GtkWidget **button,
 	{
 	GtkWidget	*b;
 
-	if (!string || !box)
+	if (!string)
 		return;
 	b = gtk_check_button_new_with_label(string);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), active);
-	gtk_box_pack_start(GTK_BOX(box), b, expand, fill, pad);
+	if (box)
+		gtk_box_pack_start(GTK_BOX(box), b, expand, fill, pad);
 	if (cb_func)
 		gtk_signal_connect(GTK_OBJECT(b), "clicked",
 				GTK_SIGNAL_FUNC(cb_func), data);
@@ -410,10 +415,11 @@ gkrellm_button_connected(GtkWidget *box, GtkWidget **button,
 	{
 	GtkWidget	*b;
 
-	if (!string || !box)
+	if (!string)
 		return;
 	b = gtk_button_new_with_label(string);
-	gtk_box_pack_start(GTK_BOX(box), b, expand, fill, pad);
+	if (box)
+		gtk_box_pack_start(GTK_BOX(box), b, expand, fill, pad);
 	if (cb_func)
 		gtk_signal_connect(GTK_OBJECT(b), "clicked",
 				GTK_SIGNAL_FUNC(cb_func), data);
@@ -553,7 +559,7 @@ create_about_tab(GtkWidget *vbox)
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 
 	buf = g_strdup_printf(_("GKrellM %d.%d.%d\nGNU Krell Monitors\n\n"
-				"Copyright (c) 1999-2001 by Bill Wilson\n"
+				"Copyright (c) 1999-2002 by Bill Wilson\n"
 				"bill@gkrellm.net\n"
 				"http://gkrellm.net\n\n"
 				"Released under the GNU Public License"),
@@ -781,7 +787,7 @@ create_general_tab(GtkWidget *tab_vbox)
 
 	vbox = gkrellm_framed_vbox_end(vbox, NULL, 4, FALSE, 0, 2);
 	gkrellm_spin_button(vbox, &update_HZ_spin_button, (gfloat) GK.update_HZ,
-			2.0, 10.0, 1.0, 1.0, 0, 55, NULL, NULL, FALSE,
+			2.0, 20.0, 1.0, 1.0, 0, 55, NULL, NULL, FALSE,
 			_("Krell and LED updates per second."));
 
 	gkrellm_spin_button(vbox, &width_spin_button, (gfloat) GK.chart_width,
@@ -813,11 +819,11 @@ static GList		*theme_position_in_list;
 static GtkWidget	*theme_clist;
 static GtkWidget	*theme_spin_button;
 static GtkWidget	*theme_alt_label;
-static GtkWidget	*default_theme_entry;
+static GtkWidget	*theme_entry,
+					*author_label;
 static GtkWidget	*font_button,
 					*fontset_button,
 					*font_preview_button;
-static gint			theme_width, author_width;
 static gboolean		theme_modified,
 					font_modified,
 					font_preview,
@@ -1017,8 +1023,10 @@ cb_theme_selected(GtkWidget *clist, gint row, gint column,
 
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(theme_spin_button), 0.0);
 	if ((theme = gtk_clist_get_row_data(GTK_CLIST(clist), row)) != NULL)
-		gtk_entry_set_text(GTK_ENTRY(default_theme_entry), theme);
-
+		{
+		gtk_entry_set_text(GTK_ENTRY(theme_entry), theme);
+		gtk_label_set_text(GTK_LABEL(author_label), get_theme_author(theme));
+		}
 	gkrellm_dup_string(&GK.theme_path, theme);
 	find_theme_position_in_list();
 	theme_modified = TRUE;
@@ -1030,13 +1038,11 @@ scan_for_themes(GtkWidget *clist)
 	{
 	GList	*list;
 	gchar	*path, *name;
-	gchar	*buf[3];
-	gint	row, l;
+	gchar	*buf[2];
+	gint	row;
 
 	make_themes_list();
-	buf[2] = NULL;
-	theme_width = 10;
-	author_width = 10;
+	buf[1] = NULL;
 
 	for (list = themes_list; list; list = list->next)
 		{
@@ -1048,14 +1054,9 @@ scan_for_themes(GtkWidget *clist)
 		buf[0] = name;
 		if (!strcmp(name, "Default"))
 			path = "";
-		buf[1] = get_theme_author(path);
 		row = gtk_clist_append(GTK_CLIST(clist), buf);
 		gtk_clist_set_row_data_full(GTK_CLIST(clist), row,
 						g_strdup(path), (GtkDestroyNotify) g_free);
-		if ((l = strlen(name)) > theme_width)
-			theme_width = l;
-		if ((l = strlen(buf[1])) > author_width)
-			author_width = l;
 		}
 	}
 
@@ -1444,7 +1445,7 @@ cb_use_fontset(GtkWidget *widget, gpointer *data)
 	}
 
 
-static gchar	*themes_titles[] = {N_("Theme"), N_("Author") };
+static gchar	*themes_titles[] = {N_("Theme")};
 
 static void
 create_theme_tab(GtkWidget *tabs_vbox)
@@ -1454,7 +1455,6 @@ create_theme_tab(GtkWidget *tabs_vbox)
 	GtkWidget		*label;
 	GtkWidget		*scrolled;
 	gchar			*s;
-	gint            i;
 
 	init_font_config();
 
@@ -1464,23 +1464,31 @@ create_theme_tab(GtkWidget *tabs_vbox)
 
 /* --Theme tab */
 	vbox = gkrellm_create_framed_tab(tabs, _("Theme"));
+
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	label = gtk_label_new(_("Theme:"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
-
-	default_theme_entry = gtk_entry_new_with_max_length(128);
+	theme_entry = gtk_entry_new_with_max_length(128);
 	if (GK.theme_path)
-		gtk_entry_set_text(GTK_ENTRY(default_theme_entry), GK.theme_path);
-	gtk_box_pack_start(GTK_BOX(hbox), default_theme_entry, TRUE, TRUE,0);
+		gtk_entry_set_text(GTK_ENTRY(theme_entry), GK.theme_path);
+	gtk_box_pack_start(GTK_BOX(hbox), theme_entry, TRUE, TRUE,0);
 
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	label = gtk_label_new(_("Author:"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
+	author_label = gtk_label_new(get_theme_author(GK.theme_path));
+	gtk_misc_set_alignment(GTK_MISC(author_label), 0, 0.5);
+	gtk_box_pack_start(GTK_BOX(hbox), author_label, TRUE, TRUE, 5);
+
+	vbox1 = gkrellm_framed_vbox(vbox, NULL, 4, TRUE, 0, 2);
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
-    for (i = 0; i < 2; i++)
-		themes_titles[i]=_(themes_titles[i]);
-	theme_clist = gtk_clist_new_with_titles(2, themes_titles);
+	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 0);
+	themes_titles[0]=_(themes_titles[0]);
+	theme_clist = gtk_clist_new_with_titles(1, themes_titles);
 	gtk_signal_connect (GTK_OBJECT(theme_clist), "select_row",
 			(GtkSignalFunc) cb_theme_selected, NULL);
 
@@ -1490,9 +1498,6 @@ create_theme_tab(GtkWidget *tabs_vbox)
 
 	scan_for_themes(theme_clist);
 
-	gtk_clist_set_column_width(GTK_CLIST(theme_clist), 0, theme_width * 7 + 8);
-	gtk_clist_set_column_width(GTK_CLIST(theme_clist), 1, author_width * 7);
-
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
 
