@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -147,13 +148,9 @@ create_hostname(GtkWidget *vbox, gint first_create)
 		yy = (d > 0) ? 0 : 1;
 		}
 
+	gkrellm_panel_configure(host, NULL, style);
 	if (decal_host || decal_sysname)
-		{
-		gkrellm_panel_configure(host, NULL, style);
 		gkrellm_panel_configure_add_height(host, yy);
-		}
-	else
-		gkrellm_panel_configure_add_height(host, window_decorations ? 5 : 3);
 
 	gkrellm_panel_create(vbox, mon_host, host);
 
@@ -165,6 +162,8 @@ create_hostname(GtkWidget *vbox, gint first_create)
 	hostname_short = GK.hostname_short;
 	system_name_visible = GK.enable_system_name;
 	draw_hostname();
+	if (!decal_host && !decal_sysname)
+		gkrellm_panel_hide(host);
 	}
 
 void
