@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -426,11 +427,11 @@ read_freebsd_tcp_port_data()
 
 			/* Ignore sockets for protocols other than tcp. */
 			if (so->xso_protocol != IPPROTO_TCP)
-				return;
+				continue;
 
 			/* Ignore PCBs which were freed during copyout. */
 			if (inp->inp_gencnt > oxig->xig_gen)
-				return;
+				continue;
 
 #if defined(INET6)
 			if (inp->inp_vflag & INP_IPV4)
@@ -1391,7 +1392,6 @@ create_inet_monitor(GtkWidget *vbox1, InetMon *in, gint first_create)
 	Chart		*cp;
 	Panel		*p;
 	Style		*style;
-	gchar		*s;
 
 	if (first_create)
 		{
@@ -1402,8 +1402,7 @@ create_inet_monitor(GtkWidget *vbox1, InetMon *in, gint first_create)
 		in->chart_hour = gkrellm_chart_new0();
 		in->panel = gkrellm_panel_new0();
 		in->chart = in->chart_minute;
-		s = g_strdup_printf(_("inet%d"), n_inet_monitors++);
-		gkrellm_dup_string(&in->name, s);
+		in->name = g_strdup_printf(_("inet%d"), n_inet_monitors++);
 		}
 	else
 		{
@@ -2169,7 +2168,7 @@ create_inet_tab(GtkWidget *tab_vbox)
 
 	vbox = gkrellm_create_framed_tab(tabs, _("Ports"));
 
-	table = gtk_table_new(7, 6, FALSE /*homogeneous*/);
+	table = gtk_table_new(6, 7, FALSE /*homogeneous*/);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 3);
 
@@ -2201,20 +2200,20 @@ create_inet_tab(GtkWidget *tab_vbox)
 	label0_entry = gtk_entry_new_with_max_length(8);
 	gtk_widget_set_usize(label0_entry, 32, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), label0_entry, 0, 1, 3, 4);
-	port0_0_entry = gtk_entry_new_with_max_length(5);
+	port0_0_entry = gtk_entry_new_with_max_length(8);
 	gtk_widget_set_usize(port0_0_entry, 32, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), port0_0_entry, 1, 2, 3, 4);
-	port0_1_entry = gtk_entry_new_with_max_length(5);
+	port0_1_entry = gtk_entry_new_with_max_length(8);
 	gtk_widget_set_usize(port0_1_entry, 32, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), port0_1_entry, 2, 3, 3, 4);
 
 	label1_entry = gtk_entry_new_with_max_length(8);
 	gtk_widget_set_usize(label1_entry, 32, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), label1_entry, 4, 5, 3, 4);
-	port1_0_entry = gtk_entry_new_with_max_length(5);
+	port1_0_entry = gtk_entry_new_with_max_length(8);
 	gtk_widget_set_usize(port1_0_entry, 32, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), port1_0_entry, 5, 6, 3, 4);
-	port1_1_entry = gtk_entry_new_with_max_length(5);
+	port1_1_entry = gtk_entry_new_with_max_length(8);
 	gtk_widget_set_usize(port1_1_entry, 32, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), port1_1_entry, 6, 7, 3, 4);
 
