@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -100,44 +101,54 @@ gkrellm_update_krell(Panel *p, Krell *k, gulong value)
 	*/
 	d = k->depth;
 
-	if (k->ema == 0 || xnew == k->x0) /* Krell can settle before ema*/
-		frame = 0;
-	else if (k->ema >= k->full_scale)
+	if (k->bar_mode)
+		{
 		frame = d - 1;
+		y_src = k->h_frame * frame;
+		x_src = k->x0;
+		x_dst = k->x0;
+		w_overlap = xnew - k->x0;
+		}
 	else
 		{
-		if (d == 1)
+		if (k->ema == 0 || xnew == k->x0) /* Krell can settle before ema*/
 			frame = 0;
-		else if (d == 2)
-			frame = 1;
+		else if (k->ema >= k->full_scale)
+			frame = d - 1;
 		else
-			frame = 1 + ((d - 2) * k->ema / k->full_scale);
+			{
+			if (d == 1)
+				frame = 0;
+			else if (d == 2)
+				frame = 1;
+			else
+				frame = 1 + ((d - 2) * k->ema / k->full_scale);
+			}
+		y_src = k->h_frame * frame;
+
+		x_src = k->x_hot - (xnew - k->x0);
+		if (x_src < 0)
+			x_src = 0;
+
+		x_dst = xnew - k->x_hot;
+		if (x_dst < k->x0)
+			x_dst = k->x0;
+
+		w1 = k->w - x_src;
+		w2 = (k->x0 + k->w_scale + 1) - x_dst;
+		w_overlap = (w2 > w1) ? w1 : w2;
+
+		/* Clear the krell stencil bitmap and draw the mask.
+		*/
+		gdk_draw_rectangle(k->stencil, GK.bit0_GC, TRUE,
+					0,0,  k->w, k->h_frame);
+		if (k->mask)
+			gdk_draw_pixmap(k->stencil, GK.bit1_GC, k->mask,
+					x_src, y_src, x_dst, 0, w_overlap, k->h_frame);
+		else
+			gdk_draw_rectangle(k->stencil, GK.bit1_GC,
+					TRUE, x_dst, 0, w_overlap, k->h_frame);
 		}
-	y_src = k->h_frame * frame;
-
-	/* Clear the krell stencil bitmap.
-	*/
-	gdk_draw_rectangle(k->stencil, GK.bit0_GC, TRUE,
-				0,0,  k->w, k->h_frame);
-
-	x_src = k->x_hot - (xnew - k->x0);
-	if (x_src < 0)
-		x_src = 0;
-
-	x_dst = xnew - k->x_hot;
-	if (x_dst < k->x0)
-		x_dst = k->x0;
-
-	w1 = k->w - x_src;
-	w2 = (k->x0 + k->w_scale + 1) - x_dst;
-	w_overlap = (w2 > w1) ? w1 : w2;
-
-	if (k->mask)
-		gdk_draw_pixmap(k->stencil, GK.bit1_GC, k->mask,
-				x_src, y_src, x_dst, 0, w_overlap, k->h_frame);
-	else
-		gdk_draw_rectangle(k->stencil, GK.bit1_GC,
-				TRUE, x_dst, 0, w_overlap, k->h_frame);
 
 	k->old_draw = k->draw;
 
@@ -156,6 +167,8 @@ gkrellm_draw_decal_on_chart(Chart *cp, Decal *d, gint x, gint y)
 	{
 	if (!cp || !d || d->state != DS_VISIBLE)
 		return;
+	d->x = x;
+	d->y = y;
 	gdk_gc_set_clip_mask(GK.text_GC, d->stencil);
 	gdk_gc_set_clip_origin(GK.text_GC, x, y);
 	gdk_draw_pixmap(cp->pixmap, GK.text_GC, d->pixmap,
@@ -478,6 +491,15 @@ gkrellm_insert_krell(Panel *p, Krell *k, gboolean append)
 	gkrellm_draw_panel_layers_force(p);
 	}
 
+void
+gkrellm_insert_krell_nth(Panel *p, Krell *k, gint n)
+	{
+	if (!p || !k || g_list_find(p->krell, k))
+		return;
+	p->krell = g_list_insert(p->krell, k, n);
+	gkrellm_draw_panel_layers_force(p);
+	}
+
 void
 gkrellm_destroy_krell_list(Panel *p)
 	{
@@ -541,7 +563,7 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 	{
 	GtkWidget	*top_win = gkrellm_get_top_window();
 	Krell		*k;
-	gint		w, h, map_x, w_render;
+	gint		w, h, w_render;
 
 	k = (Krell *) g_new0(Krell, 1);
 	if (p)
@@ -566,31 +588,31 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 		style->krell_x_hot = w / 2;
 	w_render = w;
 	k->x_hot = style->krell_x_hot;
+	k->bar_mode = FALSE;
 
-	if (style->krell_expand)
+	switch (style->krell_expand)
 		{
-		switch (style->krell_expand)
-			{
-			case KRELL_EXPAND_LEFT:
-				if (style->krell_x_hot > 0)
-					{
-					map_x = GK.chart_width;
-					w_render = map_x * w / style->krell_x_hot;
-					k->x_hot = map_x;
-					}
-				break;
-			case KRELL_EXPAND_RIGHT:
-				if (w > style->krell_x_hot)
-					{
-					map_x = GK.chart_width;
-					w_render = map_x * w / (w - style->krell_x_hot);
-					k->x_hot = style->krell_x_hot * w_render / w;
-					}
-				break;
-			}
+		case KRELL_EXPAND_NONE:
+			break;
+		case KRELL_EXPAND_BAR_MODE:
+			w_render = GK.chart_width;
+			k->bar_mode = TRUE;
+			break;
+		case KRELL_EXPAND_LEFT:
+			if (style->krell_x_hot > 0 && style->krell_x_hot < w)
+				{
+				w_render = GK.chart_width * w / style->krell_x_hot;
+				k->x_hot = GK.chart_width - 1;
+				}
+			break;
+		case KRELL_EXPAND_RIGHT:
+			if (w > style->krell_x_hot)
+				{
+				w_render = GK.chart_width * w / (w - style->krell_x_hot);
+				k->x_hot = style->krell_x_hot * w_render / w;
+				}
+			break;
 		}
-	else
-		k->x_hot = style->krell_x_hot;
 
 	gdk_imlib_render(im, w_render, h);
 
@@ -611,6 +633,17 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 		}
 
 	k->stencil = gdk_pixmap_new(top_win->window, GK.chart_width, k->h_frame,1);
+	if (k->bar_mode)
+		{	/* Stencil is never modified for bar mode, so draw it once */
+		gdk_draw_rectangle(k->stencil, GK.bit0_GC, TRUE,
+					0, 0,  k->w, k->h_frame);
+		if (k->mask)
+			gdk_draw_pixmap(k->stencil, GK.bit1_GC, k->mask,
+					0, k->h_frame * (k->depth - 1), 0, 0, k->w, k->h_frame);
+		else
+			gdk_draw_rectangle(k->stencil, GK.bit1_GC, TRUE,
+					0, 0, k->w, k->h_frame);
+		}
 
 	k->period = style->krell_ema_period;
 	if (k->period >= 4 * GK.update_HZ)
@@ -725,6 +758,15 @@ gkrellm_insert_decal(Panel *p, Decal *d, gboolean append)
 	gkrellm_draw_panel_layers_force(p);
 	}
 
+void
+gkrellm_insert_decal_nth(Panel *p, Decal *d, gint n)
+	{
+	if (!p || !d || g_list_find(p->decal, d))
+		return;
+	p->decal = g_list_insert(p->decal, d, n);
+	gkrellm_draw_panel_layers_force(p);
+	}
+
 void
 gkrellm_destroy_decal_list(Panel *p)
 	{
@@ -858,7 +900,7 @@ gkrellm_create_decal_text(Panel *p, gchar *string, TextStyle *ts,
 void
 gkrellm_make_decal_invisible(Panel *p, Decal *d)
 	{
-	if (!p || !d)
+	if (!p || !d || d->state == DS_INVISIBLE)
 		return;
 	d->state = DS_ERASING;
 	d->modified = TRUE;
@@ -869,7 +911,7 @@ gkrellm_make_decal_invisible(Panel *p, Decal *d)
 void
 gkrellm_make_decal_visible(Panel *p, Decal *d)
 	{
-	if (!p || !d)
+	if (!p || !d || d->state == DS_VISIBLE)
 		return;
 	d->state = DS_VISIBLE;
 	d->modified = TRUE;
@@ -888,8 +930,8 @@ gkrellm_is_decal_visible(Decal *d)
 #define	OVERLAY_BUTTON 	1
 
 
-static gint
-in_button(Decal *d, GdkEventButton *ev)
+gboolean
+gkrellm_in_decal(Decal *d, GdkEventButton *ev)
 	{
 	if (!d || !ev)
 		return FALSE;
@@ -911,6 +953,29 @@ set_button_index(DecalButton *b, gint index, gint do_draw)
 		gkrellm_draw_panel_layers(b->p);
 	}
 
+void
+gkrellm_set_button_sensitive(DecalButton *b, gboolean sensitive)
+	{
+	if (b)
+		b->sensitive = sensitive;
+	}
+
+void
+gkrellm_hide_button(DecalButton *b)
+	{
+	if (!b)
+		return;
+	b->sensitive = FALSE;
+	gkrellm_make_decal_invisible(b->p, b->d);
+	}
+
+void
+gkrellm_show_button(DecalButton *b)
+	{
+	b->sensitive = TRUE;
+	gkrellm_make_decal_visible(b->p, b->d);
+	}
+
   /* FIXME, button_press and button_release are called multiple times
   |  for each real event if there are multiple buttons in the panel.
   |  This is because I set signal handlers for each button.  Signal
@@ -922,20 +987,23 @@ cb_decal_button_press(GtkWidget *widget, GdkEventButton *ev)
 	GList		*list;
 	DecalButton	*b;
 
-	if (ev->button != 1)
-		return TRUE;
 	for (list = button_list; list; list = list->next)
 		{
 		b = (DecalButton *) list->data;
 		if (widget != b->p->drawing_area)
 			continue;
-		if (b->sensitive && (*(b->cb_in_button))(b->d, ev))
+		if (   b->sensitive && (*(b->cb_in_button))(b->d, ev)
+			&& (   (b->cb_button_click && ev->button == 1)
+				|| (b->cb_button_right_click && ev->button == 3)
+			   )
+		   )
 			{
 			if (b->cur_index != b->pressed_index)
 				{
 				b->saved_index = b->cur_index;
 				set_button_index(b, b->pressed_index, 1);
 				}
+			break;
 			}
 		}
 	return TRUE;
@@ -946,6 +1014,7 @@ cb_decal_button_release(GtkWidget *widget, GdkEventButton *ev)
 	{
 	GList		*list;
 	DecalButton	*b;
+	gboolean	in_button;
 
 	for (list = button_list; list; list = list->next)
 		{
@@ -955,8 +1024,11 @@ cb_decal_button_release(GtkWidget *widget, GdkEventButton *ev)
 		if (b->cur_index == b->pressed_index)
 			{
 			set_button_index(b, b->saved_index, 1);
-			if ((*(b->cb_in_button))(b->d, ev) && b->cb_button_event)
-				(*(b->cb_button_event))(b);
+			in_button = (*(b->cb_in_button))(b->d, ev);
+			if (in_button && b->cb_button_click && ev->button == 1)
+				(*(b->cb_button_click))(b, b->data);
+			else if (in_button && b->cb_button_right_click && ev->button == 3)
+				(*(b->cb_button_right_click))(b, b->right_data);
 		 	}
 		}
 	return TRUE;
@@ -1083,6 +1155,23 @@ gkrellm_set_in_button_callback(DecalButton *b, gint (*func)())
 		b->cb_in_button = func;
 	}
 
+void
+gkrellm_decal_button_connect(DecalButton *b, void (*func)(), void *data)
+	{
+	if (!b)
+		return;
+	b->data = data;
+	b->cb_button_click = func;
+	}
+
+void
+gkrellm_decal_button_right_connect(DecalButton *b, void (*func)(), void *data)
+	{
+	if (!b)
+		return;
+	b->right_data = data;
+	b->cb_button_right_click = func;
+	}
 
   /* Make an existing decal into a decal button.  The decal should already
   |  be created and be in the panel's decal list.
@@ -1100,8 +1189,8 @@ gkrellm_make_decal_button(Panel *p, Decal *d, void (*func)(), void *data,
 	b->d = d;
 	b->pressed_index = pressed_index;
 	b->data = data;
-	b->cb_button_event = func;
-	b->cb_in_button = in_button;
+	b->cb_button_click = func;
+	b->cb_in_button = gkrellm_in_decal;
 	b->sensitive = TRUE;
 
 	b->id_press = gtk_signal_connect(GTK_OBJECT(p->drawing_area),
