@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -119,7 +120,7 @@ static GdkGC	*trans_gc;
 
 static gchar	*geometry;
 
-static gint		x_position, y_position;
+static gint		x_position, y_position, y_pack;
 
 static gint		withdrawn;
 static gint		total_frame_height;
@@ -133,6 +134,7 @@ static gint		no_transparency;
 static void		apply_rootpixmap_transparency(void);
 static void		apply_frame_transparency(void);
 
+
 #define	N_FALLBACK_FONTS	3
 
 static gchar	*fail_large_font[N_FALLBACK_FONTS] =
@@ -579,14 +581,14 @@ x_label_position(gint position, gint w_field, gint w_label, gint margin)
 	return x;
 	}
 
-static void
+gboolean
 draw_rootpixmap_onto_transparent_chart(Chart *cp)
 	{
 	Window	child;
 	gint	x, y;
 
-	if (root_xpixmap == None)
-		return;
+	if (root_xpixmap == None || !cp->transparency)
+		return FALSE;
 	XTranslateCoordinates(GDK_DISPLAY(),
 			GDK_WINDOW_XWINDOW(cp->drawing_area->window),
 			GDK_ROOT_WINDOW(),
@@ -607,6 +609,7 @@ draw_rootpixmap_onto_transparent_chart(Chart *cp)
 				cp->bg_clean_pixmap, 0, 0, 0, 0, cp->w, cp->h);
 		gdk_gc_set_clip_mask(GK.text_GC, NULL);
 		}
+	return TRUE;
 	}	
 
 gboolean
@@ -745,8 +748,9 @@ gkrellm_motion(GtkWidget *widget, GdkEventMotion *ev, gpointer data)
 		/* Moves to x,y position relative to root.  Subtract the press
 		|  event coordinates to account for pointer offset into top_window.
 		*/
+		y_position = y_pointer - y_press_event;
 		gdk_window_move(top_window->window,
-			x_pointer - x_press_event, y_pointer - y_press_event);
+					x_pointer - x_press_event, y_position);
 		}
 	}
 
@@ -1073,7 +1077,7 @@ gkrellm_pack_side_frames(void)
 	{
 	static GtkWidget	*lf_pixmapwid, *rf_pixmapwid;
 	gint				x_gkrell, y_gkrell, y_bottom;
-	gint				y, w, h;
+	gint				w, h;
 	gint				was_on_bottom;
 
 	if (   ! GK.initialized
@@ -1110,26 +1114,40 @@ gkrellm_pack_side_frames(void)
 				FALSE, FALSE, 0);	
 	gtk_widget_show(rf_pixmapwid);
 
-
-	/* If GKrellM grows in height and bottom is moved off screen, move so
-	|  that all of GKrellM is visible.
+	/* If height changed so that all of GKrellM would be visible if at last
+	|  user set y_position, then make sure we are really at the user set
+	|  y_position.   This is a y memory effect in case a previous
+	|  side frame packing caused a y move.
 	*/
 	h = GK.monitor_height + total_frame_height;
-	y_bottom = h_display - (y_gkrell + h);
-	if (y_bottom < 0)
+	y_bottom = h_display - (y_position + h);
+	if (y_bottom >= 0)
 		{
-		if ((y = y_gkrell + y_bottom) < 0)
-			y = 0;
-		gdk_window_move(gtree.window->window, x_gkrell, y);
+		if (y_position != y_gkrell)
+			gdk_window_move(gtree.window->window, x_gkrell, y_position);
+		y_pack = -1;
 		}
-	/* If GKrellM bottom edge was <= screen bottom, then move to make
-	|  resized GKrellM still on bottom whether window has shrunk or grown.
-	*/
-	else if (was_on_bottom)
+	else	/* Otherwise, do a y move to maximize visibility */
 		{
-		if ((y = h_display - h) < 0)
-			y = 0;
-		gdk_window_move(gtree.window->window, x_gkrell, y);
+		/* If GKrellM grows in height and bottom is moved off screen, move so
+		|  that all of GKrellM is visible.
+		*/
+		y_bottom = h_display - (y_gkrell + h);
+		if (y_bottom < 0)
+			{
+			if ((y_pack = y_gkrell + y_bottom) < 0)
+				y_pack = 0;
+			gdk_window_move(gtree.window->window, x_gkrell, y_pack);
+			}
+		/* If GKrellM bottom edge was <= screen bottom, then move to make
+		|  resized GKrellM still on bottom whether window has shrunk or grown.
+		*/
+		else if (was_on_bottom)
+			{
+			if ((y_pack = h_display - h) < 0)
+				y_pack = 0;
+			gdk_window_move(gtree.window->window, x_gkrell, y_pack);
+			}
 		}
 	monitor_previous_height = GK.monitor_height;
 	check_rootpixmap_transparency = 3;
@@ -1153,7 +1171,7 @@ edge_record()
 static void
 fix_edges()
 	{
-	gint	x, y, w, h, x0, y0;
+	gint	x, y, w, h, x0, y0, y_bottom;
 
 	gdk_window_get_origin(gtree.window->window, &x, &y);
 	w = GK.chart_width + GK.frame_left_width + GK.frame_right_width;
@@ -1169,8 +1187,21 @@ fix_edges()
 		y = 0;
 	if (y > h_display - h || on_edge[3])
 		y = h_display - h;
+
+	/* Make sure we bias to last user set y_position
+	*/
+	y_bottom = h_display - (y_position + h);
+	if (y_bottom >= 0)
+		y = y_position;
 	if (x != x0 || y != y0)
+		{
+		/* A theme change move adjustment to keep all visible, but treat this
+		|  as a packing move so we can bias to the last user set y_position.
+		*/
 		gdk_window_move(gtree.window->window, x, y);
+		if (y != y_position)
+			y_pack = y;
+		}
 	}
 
   /* Use XParseGeometry, but width and height are ignored.
@@ -1191,6 +1222,7 @@ place_gkrellm(gchar *geom)
 	if (place & XNegative)
 		x = w_display - w_gkrell + x;
 	gdk_window_move(gtree.window->window, x, y);
+	y_position = y;
 	}
 
 static void
@@ -1427,7 +1459,7 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 
 	if (x >= 0 && x < w_display - 10)
 		x_position = x;
-	if (y >= 0 && y < h_display - 25)
+	if (y >= 0 && y < h_display - 5 && y != y_pack)
 		y_position = y;
 
 	/* If GKrellM is killed (signal, X shutdown, etc) it might not get
@@ -1621,7 +1653,7 @@ build_gkrellm()
 	total_frame_height = 0;
 	root_xpixmap = None;
 	GK.any_transparency = FALSE;
-	start_timer(0);				/* Stop the timer */
+	start_timer(0);
 
 	if (!first_create)
 		edge_record();
@@ -1639,9 +1671,6 @@ build_gkrellm()
 
 	create_frame_top(gtree.top0_vbox);
 
-	/* Create the monitors.  Each monitor must increment GK.monitor_height
-	|  by its vertical space.
-	*/
 	GK.monitor_height = 0;
 	create_hostname(gtree.top1_vbox, first_create);
 	for (list = gkrellm_monitor_list; list; list = list->next)
@@ -1702,10 +1731,6 @@ add_builtin(Monitor *mon)
 	mon->privat->enabled = TRUE;
 	}
 
-  /* Eventually, add a config for selecting specific monitors here so
-  |  multiple gkrellm instances can be set up with distinct monitor
-  |  sets.
-  */
 static void
 load_builtin_monitors()
 	{
@@ -1765,7 +1790,7 @@ int
 main (int argc, char *argv[])
 	{
 	gint		i, on_top = 0;
-	gchar		*s;
+	gchar		*s, *sm;
 
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 	/* We just ignore error, here.  Even if GKrellM doesn't have
@@ -1871,9 +1896,16 @@ main (int argc, char *argv[])
 			if (GK.sm_argc < GKRELLM_SM_ARGC_MAX - 1)
 				{
 				GK.sm_argv[GK.sm_argc++] = argv[i];
-				GK.sm_argv[GK.sm_argc++] = argv[i + 1];
+				if (i < argc - 1)
+					{
+					sm = argv[i + 1];
+					if (*sm != '-')
+						{
+						GK.sm_argv[GK.sm_argc++] = argv[i + 1];
+						++i;
+						}
+					}
 				}
-			++i;
 			}
 		else if ((!strcmp(s, "t") || !strcmp(s, "theme")) && i < argc - 1)
 			{
