@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -537,12 +538,25 @@ read_freebsd_meminfo()
 
 #define	PROC_MEMINFO_FILE	"/proc/meminfo"
 
+  /* Kernels >= 2.5.x have tagged formats only in kb units.
+  */
+static void
+tagged_format_meminfo(gchar *buf, memint *mint)
+	{
+#if defined(G_HAVE_GINT64)
+	sscanf(buf,"%*s %Lu", mint);
+#else
+	sscanf(buf,"%*s %lu", mint);
+#endif
+	*mint *= 1024;
+	}
 
 static void
 read_proc_meminfo(void)
 	{
 	FILE		*f;
 	gchar		buf[160];
+	gboolean	using_tagged = FALSE;
 
 	/* Reading /proc/meminfo is expensive under Linux kernel 2.2, so
 	|  take extra precautions to minimize reading it.  If kernel 2.4
@@ -554,27 +568,53 @@ read_proc_meminfo(void)
 			{		/* total:    used:    free:  shared: buffers:  cached: */
 			while ((fgets(buf, sizeof(buf), f)) != NULL)
 				{
-				if (strncmp(buf, "Mem:", 4) == 0)
-#if defined(G_HAVE_GINT64)
-					sscanf(buf,"Mem: %Lu %Lu %Lu %Lu %Lu %Lu",
-						&mem.total, &mem.x_used, &mem.free,
-						&mem.shared, &mem.buffers, &mem.cached);
-#else
-					sscanf(buf,"Mem: %lu %lu %lu %lu %lu %lu",
-						&mem.total, &mem.x_used, &mem.free,
-						&mem.shared, &mem.buffers, &mem.cached);
-#endif
-				if (strncmp(buf, "Swap:", 5) == 0)
+				if (buf[0] == 'M')
 					{
+					if (!strncmp(buf, "Mem:", 4))
 #if defined(G_HAVE_GINT64)
-					sscanf(buf,"Swap: %Lu %Lu", &swap.total,
-						&swap.used);
+						sscanf(buf,"Mem: %Lu %Lu %Lu %Lu %Lu %Lu",
+							&mem.total, &mem.x_used, &mem.free,
+							&mem.shared, &mem.buffers, &mem.cached);
 #else
-					sscanf(buf,"Swap: %lu %lu", &swap.total,
-						&swap.used);
+						sscanf(buf,"Mem: %lu %lu %lu %lu %lu %lu",
+							&mem.total, &mem.x_used, &mem.free,
+							&mem.shared, &mem.buffers, &mem.cached);
 #endif
-					break;
+					else if (!strncmp(buf, "MemTotal:", 9))
+						{
+						tagged_format_meminfo(buf, &mem.total);
+						using_tagged = TRUE;
+						}
+					else if (!strncmp(buf, "MemFree:", 8))
+						tagged_format_meminfo(buf, &mem.free);
+					else if (!strncmp(buf, "MemShared:", 10))
+						tagged_format_meminfo(buf, &mem.shared);
 					}
+				else if (buf[0] == 'S')
+					{
+					if (!strncmp(buf, "Swap:", 5))
+						{
+#if defined(G_HAVE_GINT64)
+						sscanf(buf,"Swap: %Lu %Lu", &swap.total, &swap.used);
+#else
+						sscanf(buf,"Swap: %lu %lu", &swap.total, &swap.used);
+#endif
+						break;
+						}
+					else if (!strncmp(buf, "SwapTotal:", 10))
+						tagged_format_meminfo(buf, &swap.total);
+					else if (!strncmp(buf, "SwapFree:", 9))
+						tagged_format_meminfo(buf, &swap.used);
+					}
+				else if (buf[0] == 'B' && !strncmp(buf, "Buffers:", 8))
+					tagged_format_meminfo(buf, &mem.buffers);
+				else if (buf[0] == 'C' && !strncmp(buf, "Cached:", 7))
+					tagged_format_meminfo(buf, &mem.cached);
+				}
+			if (using_tagged)
+				{
+				mem.x_used = mem.total - mem.free;
+				swap.used = swap.total - swap.used;
 				}
 			mem.used = mem.x_used - mem.buffers - mem.cached;
 			fclose(f);
@@ -1376,10 +1416,16 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 	Panel			*p;
 	TextStyle		*ts;
 	Style			*s;
+	gchar			*expand;
 
 	mm = &mem;
 	if (first_create)
 		mm->panel = gkrellm_panel_new0();
+	if (!mem.all_krells)	/* Krells are not in panel krell list where */
+		{					/* they would be automatically destroyed. */
+		gkrellm_destroy_krell(mem.krell_buffers);
+		gkrellm_destroy_krell(mem.krell_cache);
+		}
 	p = mm->panel;
 
 	/* I've got two extra krells for buffers and cache.  Use the #included
@@ -1392,6 +1438,9 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 	gkrellm_get_gkrellmrc_integer("mem_krell_buffers_depth", &s->krell_depth);
 	gkrellm_get_gkrellmrc_integer("mem_krell_buffers_x_hot", &s->krell_x_hot);
 	gkrellm_get_gkrellmrc_integer("mem_krell_buffers_yoff", &s->krell_yoff);
+	expand = gkrellm_get_gkrellmrc_string("mem_krell_buffers_expand");
+	gkrellm_set_krell_expand(s, expand);
+	g_free(expand);
 	gkrellm_load_image("krell_buffers", krell_buffers_xpm, &im,MEM_STYLE_NAME);
 	mm->krell_buffers = gkrellm_create_krell(p, im, s);
 	gkrellm_monotonic_krell_values(mm->krell_buffers, FALSE);
@@ -1402,6 +1451,9 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 	gkrellm_get_gkrellmrc_integer("mem_krell_cache_depth", &s->krell_depth);
 	gkrellm_get_gkrellmrc_integer("mem_krell_cache_x_hot", &s->krell_x_hot);
 	gkrellm_get_gkrellmrc_integer("mem_krell_cache_yoff", &s->krell_yoff);
+	expand = gkrellm_get_gkrellmrc_string("mem_krell_cache_expand");
+	gkrellm_set_krell_expand(s, expand);
+	g_free(expand);
 	gkrellm_load_image("krell_cache", krell_cache_xpm, &im, MEM_STYLE_NAME);
 	mm->krell_cache = gkrellm_create_krell(p, im, s);
 	gkrellm_monotonic_krell_values(mm->krell_cache, FALSE);
