@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -61,8 +62,17 @@ typedef struct
 
 	gint		extra_info;
 	gchar		*label;
-	gulong		oldrx,
-				oldtx;
+	gulong		rx_old,
+				tx_old;
+	gint		rx_current,
+				tx_current;
+	gdouble		rx_totalA,
+				rx_totalB,
+				tx_totalA,
+				tx_totalB;
+	gboolean	show_totalB,
+				totals_shown,
+				mouse_in_chart;
 	Launcher	launch;
 	GtkWidget	*launch_entry,
 				*tooltip_entry;
@@ -884,6 +894,15 @@ static GdkBitmap	*decal_timer_button_mask;
 static Decal		*time_decal,
 					*seconds_decal,
 					*button_decal;
+
+  /* These 3 decals will be drawn on net charts when mouse is in a chart and
+  |  the chart has a rx and/or tx total drawn.  They won't live in a panel
+  |  so must be managed differently from decals on a panel.
+  */
+static Decal		*decal_totalA,
+					*decal_totalB,
+					*decal_reset;
+
 static gchar		*timer_on_command;
 static gchar		*timer_off_command;
 static gchar		*timer_button_iface;
@@ -1126,7 +1145,7 @@ static gint	save_tb_state;
 static void
 cb_timer_button_press(GtkWidget *widget, GdkEventButton *ev)
 	{
-	if (! in_button(DECAL(timer_panel), ev))
+	if (ev->button != 1 || !in_button(DECAL(timer_panel), ev))
 		return;
 	if (timer_button_state != TB_PRESSED)		/* button bounce? */
 		save_tb_state = timer_button_state;
@@ -1423,6 +1442,7 @@ destroy_net_monitor(NetMon *net)
 		{
 		net->launch.tooltip = NULL;
 		g_free(cp->panel->textstyle);
+		cp->panel->textstyle = NULL;
 		gkrellm_chart_destroy(cp);
 		gtk_widget_destroy(net->vbox);
 		net->chart = NULL;
@@ -1433,7 +1453,7 @@ destroy_net_monitor(NetMon *net)
 	}
 
 
-static size_abbrev_table	net_bytes_abbrev[]	=
+static size_abbrev_table	current_bytes_abbrev[]	=
 	{
 	{ KB_SIZE(1),		1,				"%.0f" },
 	{ KB_SIZE(20),		KB_SIZE(1),		"%.1fK" },
@@ -1446,6 +1466,21 @@ static size_abbrev_table	net_bytes_abbrev[]	=
 	{ TB_SIZE(1000),	TB_SIZE(1),		"%.0fT" }
 	};
 
+static size_abbrev_table	total_bytes_abbrev[]	=
+	{
+	{ KB_SIZE(100),		1,				"%.0f" },
+	{ MB_SIZE(1),		KB_SIZE(1),		"%.1fK" },
+	{ MB_SIZE(10),		MB_SIZE(1),		"%.3fM" },
+	{ MB_SIZE(100),		MB_SIZE(1),		"%.2fM" },
+	{ GB_SIZE(1),		MB_SIZE(1),		"%.1fM" },
+	{ GB_SIZE(10),		GB_SIZE(1),		"%.3fG" },
+	{ GB_SIZE(100),		GB_SIZE(1),		"%.2fG" },
+	{ TB_SIZE(1),		GB_SIZE(1),		"%.1fG" },
+	{ TB_SIZE(10),		TB_SIZE(1),		"%.3fT" },
+	{ TB_SIZE(100),		TB_SIZE(1),		"%.2fT" },
+	{ TB_SIZE(1000),	TB_SIZE(1),		"%.1fT" }
+	};
+
 #define	DEFAULT_TEXT_FORMAT	"$T\\b\\c\\f$L"
 
 static gchar    *text_format;
@@ -1455,29 +1490,35 @@ format_chart_text(NetMon *net, gchar *buf, gint size)
 	{
 	Chart	*cp;
 	gchar	c, *s, *s1;
-	size_t	tbl_size;
-	gint	len, rx_bytes, tx_bytes, bytes;
+	gint	len, bytes;
+	gdouble	fbytes;
 
 	--size;
 	*buf = '\0';
 	cp = net->chart;
-	rx_bytes = gkrellm_get_current_chartdata(net->rx_cd);
-	tx_bytes = gkrellm_get_current_chartdata(net->tx_cd);
-	tbl_size = sizeof(net_bytes_abbrev) / sizeof(size_abbrev_table);
+	net->totals_shown = FALSE;
 	for (s = text_format; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
 		if (*s == '$' && *(s + 1) != '\0')
 			{
 			bytes = -1;
+			fbytes = -1.0;
 			if ((c = *(s + 1)) == 'T')
-				bytes = rx_bytes + tx_bytes;
+				bytes = net->rx_current + net->tx_current;
 			else if (c == 'M')
 				bytes = gkrellm_get_chart_scalemax(cp);
 			else if (c == 't')
-				bytes = tx_bytes;
+				bytes = net->tx_current;
 			else if (c == 'r')
-				bytes = rx_bytes;
+				bytes = net->rx_current;
+			else if (c == 'o')
+				fbytes = net->show_totalB ? net->tx_totalB : net->tx_totalA;
+			else if (c == 'i')
+				fbytes = net->show_totalB ? net->rx_totalB : net->rx_totalA;
+			else if (c == 'O')
+				fbytes = net->show_totalB ? (net->rx_totalB + net->tx_totalB)
+							: (net->rx_totalA + net->tx_totalA);
 			else if (c == 'L')
 				{
 				if (!net->label || *(net->label) == '\0')
@@ -1497,7 +1538,15 @@ format_chart_text(NetMon *net, gchar *buf, gint size)
 				}
 			if (bytes >= 0)
 				len = format_size_abbrev(buf, size, (gfloat) bytes,
-						&net_bytes_abbrev[0], tbl_size);
+					&current_bytes_abbrev[0],
+					sizeof(current_bytes_abbrev) / sizeof(size_abbrev_table));
+			else if (fbytes >= 0)
+				{
+				len = format_size_abbrev(buf, size, (gfloat) fbytes,
+					&total_bytes_abbrev[0],
+					sizeof(total_bytes_abbrev) /sizeof(size_abbrev_table));
+				net->totals_shown = TRUE;
+				}
 			++s;
 			}
 		else
@@ -1520,41 +1569,98 @@ draw_net_extra(NetMon *net)
 	gkrellm_draw_chart_text(cp, net_style_id, buf);
 	}
 
+static gboolean
+draw_totals_buttons(NetMon *net)
+	{
+	gint	frame, x, y;
+
+	if (!net->totals_shown || !net->extra_info)
+		return FALSE;
+	x = gkrellm_chart_width() - 2 * decal_totalA->w;
+	y = 2;
+
+	frame = net->show_totalB ? D_MISC_BUTTON_OUT : D_MISC_BUTTON_IN;
+	gkrellm_draw_decal_pixmap(NULL, decal_totalA, frame);
+	gkrellm_draw_decal_on_chart(net->chart, decal_totalA, x, y);
+
+	frame = net->show_totalB ? D_MISC_BUTTON_IN : D_MISC_BUTTON_OUT;
+	gkrellm_draw_decal_pixmap(NULL, decal_totalB, frame);
+	gkrellm_draw_decal_on_chart(net->chart, decal_totalB,
+				x + decal_totalA->w, y);
+
+	gkrellm_draw_decal_pixmap(NULL, decal_reset, D_MISC_BUTTON_OUT);
+	gkrellm_draw_decal_on_chart(net->chart, decal_reset,
+				x + decal_totalA->w, y + decal_totalA->h + 4);
+	return TRUE;
+	}
+
 static void
 refresh_net_chart(NetMon *net)
 	{
 	gkrellm_draw_chartdata(net->chart);
 	draw_net_extra(net);
+	if (net->mouse_in_chart)
+		draw_totals_buttons(net);
 	gkrellm_draw_chart_to_screen(net->chart);
 	}
 
 
 static gint
-cb_net_extra(GtkWidget *widget, GdkEventButton *ev)
+cb_chart_press(GtkWidget *widget, GdkEventButton *ev, NetMon *net)
 	{
-	GList		*list;
-	NetMon		*net;
 	NetConfig	*nc;
+	gboolean	check_button;
 
-	for (list = net_mon_list; list; list = list->next)
+	check_button = net->extra_info && net->totals_shown;
+	if (check_button && gkrellm_in_decal(decal_totalA, ev))
 		{
-		net = (NetMon *) list->data;
-		if (net->created == FALSE || net->chart->drawing_area != widget)
-			continue;
-		if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS)
-			{
-			net->extra_info = !net->extra_info;
-			if ((nc = lookup_net_config(net->name)) != NULL)
-				nc->extra_info = net->extra_info;
-			gkrellm_config_modified();
-			refresh_net_chart(net);
-			}
-		else if (   ev->button == 3
-				 || (ev->button == 1 && ev->type == GDK_2BUTTON_PRESS)
-				)
-			gkrellm_chartconfig_window_create(net->chart);
-		break;
+		net->show_totalB = FALSE;
+		refresh_net_chart(net);
 		}
+	else if (check_button && gkrellm_in_decal(decal_totalB, ev))
+		{
+		net->show_totalB = TRUE;
+		refresh_net_chart(net);
+		}
+	else if (check_button && gkrellm_in_decal(decal_reset, ev))
+		{
+		if (net->show_totalB)
+			net->rx_totalB = net->tx_totalB = 0;
+		else
+			net->rx_totalA = net->tx_totalA = 0;
+		refresh_net_chart(net);
+		}
+	else if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS)
+		{
+		net->extra_info = !net->extra_info;
+		if ((nc = lookup_net_config(net->name)) != NULL)
+			nc->extra_info = net->extra_info;
+		gkrellm_config_modified();
+		refresh_net_chart(net);
+		}
+	else if (   ev->button == 3
+			 || (ev->button == 1 && ev->type == GDK_2BUTTON_PRESS)
+			)
+			gkrellm_chartconfig_window_create(net->chart);
+
+	return TRUE;
+	}
+
+static gint
+cb_chart_enter(GtkWidget *w, GdkEventButton *ev, NetMon *net)
+	{
+	net->mouse_in_chart = TRUE;
+	if (draw_totals_buttons(net))
+		gkrellm_draw_chart_to_screen(net->chart);
+	return TRUE;
+	}
+
+static gint
+cb_chart_leave(GtkWidget *w, GdkEventButton *ev, NetMon *net)
+	{
+	net->mouse_in_chart = FALSE;
+	if (net->totals_shown)
+		refresh_net_chart(net);
 	return TRUE;
 	}
 
@@ -1638,14 +1744,21 @@ create_net_monitor(GtkWidget *vbox, NetMon *net, gint first_create)
 		{
 		net->extra_info = nc->extra_info;
 		net->force_net_up = nc->force_net_up;
+
 		gtk_signal_connect(GTK_OBJECT (cp->drawing_area), "expose_event",
 				(GtkSignalFunc) net_expose_event, NULL);
+		gtk_signal_connect(GTK_OBJECT(cp->drawing_area),
+				"button_press_event", (GtkSignalFunc) cb_chart_press, net);
+		gtk_signal_connect(GTK_OBJECT(cp->drawing_area), "enter_notify_event",
+				(GtkSignalFunc) cb_chart_enter, net);
+		gtk_signal_connect(GTK_OBJECT(cp->drawing_area), "leave_notify_event",
+				(GtkSignalFunc) cb_chart_leave, net);
+
 		gtk_signal_connect(GTK_OBJECT (p->drawing_area), "expose_event",
 				(GtkSignalFunc) net_expose_event, NULL);
-		gtk_signal_connect(GTK_OBJECT(cp->drawing_area),
-				"button_press_event", (GtkSignalFunc) cb_net_extra, NULL);
 		gtk_signal_connect(GTK_OBJECT(p->drawing_area),
 				"button_press_event", (GtkSignalFunc) cb_panel_press, NULL);
+
 		gtk_widget_show_all(net->vbox);
 		}
 	else
@@ -1801,20 +1914,26 @@ update_net(void)
 			continue;
 		if (net->state != NET_DOWN)
 			{
-			if (net->rx > net->oldrx)
+			if (net->rx > net->rx_old)
 				draw_led(net, RX_LED, RX_ON);
 			else
 				draw_led(net, RX_LED, RX_OFF);
-			if (net->tx > net->oldtx)
+			if (net->tx > net->tx_old)
 				draw_led(net, TX_LED, TX_ON);
 			else
 				draw_led(net, TX_LED, TX_OFF);
 			}
-		net->oldrx = net->rx;
-		net->oldtx = net->tx;
+		net->rx_old = net->rx;
+		net->tx_old = net->tx;
 		if (GK.second_tick)
 			{
 			gkrellm_store_chartdata(net->chart, 0, net->tx, net->rx);
+			net->rx_current = gkrellm_get_current_chartdata(net->rx_cd);
+			net->tx_current = gkrellm_get_current_chartdata(net->tx_cd);
+			net->rx_totalA += (gdouble) net->rx_current;
+			net->tx_totalA += (gdouble) net->tx_current;
+			net->rx_totalB += (gdouble) net->rx_current;
+			net->tx_totalB += (gdouble) net->tx_current;
 			refresh_net_chart(net);
 			}
 		/* I don't care if this unsigned long add overflows because I
@@ -2012,6 +2131,25 @@ create_net(GtkWidget *vbox, gint first_create)
 			   )
 				create_net_monitor(NULL, net, first_create);
 			}
+	/* Some decals that don't live in a panel, they will be drawn onto
+	|  charts when needed.  So, must destroy them at create events (not done
+	|  automatically in panel lists) and then recreate them with NULL panel
+	|  and style pointers.
+	*/
+	gkrellm_destroy_decal(decal_totalA);
+	gkrellm_destroy_decal(decal_totalB);
+	gkrellm_destroy_decal(decal_reset);
+	decal_totalA = gkrellm_create_decal_pixmap(NULL,
+			gkrellm_decal_misc_pixmap(), gkrellm_decal_misc_mask(),
+			N_MISC_DECALS, NULL, 0, 0);
+	decal_totalB = gkrellm_create_decal_pixmap(NULL,
+			gkrellm_decal_misc_pixmap(), gkrellm_decal_misc_mask(),
+			N_MISC_DECALS, NULL, 0, 0);
+
+	decal_reset = gkrellm_create_decal_pixmap(NULL,
+			gkrellm_decal_misc_pixmap(), gkrellm_decal_misc_mask(),
+			N_MISC_DECALS, NULL, 0, 0);
+
 	create_net_timer(timer_vbox, first_create);
 	ascent = 0;
 	ascent_alt = 0;
@@ -2229,9 +2367,12 @@ N_("Routed net interfaces are automatically detected and charted.\n"
 N_("<b>Chart Labels\n"),
 N_("Substitution variables for the format string for chart labels:\n"),
 N_("\t$M    maximum chart value\n"),
-N_("\t$T    total receive + transmit bytes\n"),
+N_("\t$T    receive + transmit bytes\n"),
 N_("\t$r    receive bytes\n"),
 N_("\t$t    transmit bytes\n"),
+N_("\t$O    cumulative receive + transmit bytes\n"),
+N_("\t$i    cumulative receive bytes\n"),
+N_("\t$o    cumulative transmit bytes\n"),
 N_("\t$L    the optional chart label\n")
 }
 ;
@@ -2356,6 +2497,8 @@ create_net_tab(GtkWidget *tab_vbox)
 	list = g_list_append(list, "\\c\\f$M\\n$T\\b\\c\\f$L");
 	list = g_list_append(list,
 		_("\\f\\ww\\c\\f$M\\n\\f\\at\\.$t\\n\\f\\ar\\.$r\\b\\c\\f$L"));
+	list = g_list_append(list,
+		_("\\f\\ww\\c\\f$M\\n\\f\\at\\.$o\\n\\f\\ar\\.$i\\b\\c\\f$L"));
 	list = g_list_append(list,
 		_("\\f\\ww\\c\\f$M\\D2\\f\\ar\\.$r\\D1\\f\\at\\.$t\\b\\c\\f$L"));
 	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
