@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -922,7 +923,9 @@ load_plugins_enable_file(void)
 			{
 			if ((s = strchr(buf, '\n')) != NULL)
 				*s = '\0';
-			s = g_strdup(buf);
+			if (!buf[0])
+				continue;
+			s = g_strdup(g_basename(buf));
 			plugins_enable_list = g_list_append(plugins_enable_list, s);
 			}
 		fclose(f);
@@ -948,7 +951,7 @@ save_plugins_enable_file(void)
 			{
 			m = (Monitor *) list->data;
 			if (m->privat->enabled && !m->privat->from_command_line)
-				fprintf(f, "%s\n", m->path);
+				fprintf(f, "%s\n", g_basename(m->path));
 			}
 		fclose(f);
 		}
@@ -993,7 +996,7 @@ scan_for_plugins(gchar *path)
 			if (m)
 				{
 				plugins_list = g_list_append(plugins_list, m);
-				if (! string_in_list(plugins_enable_list, m->path))
+				if (! string_in_list(plugins_enable_list, g_basename(m->path)))
 					m->privat->enabled = FALSE;
 				}
 			else
