@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -36,6 +37,7 @@
 
 #include "gkrellm.h"
 #include "gkrellm_private_proto.h"
+#include "gkrellm_threads.h"
 
 #include <dirent.h>
 
@@ -93,7 +95,8 @@ typedef struct _sensor
 	GtkWidget	*alert_button;
 	GtkWidget	*vref_entry;
 
-	gfloat		value;
+	gfloat		value,
+				raw_value;
 	gboolean	value_valid;
 
 	Alert		*alert;
@@ -297,7 +300,7 @@ get_freebsd_sensor(Sensor *sensor)
 		{
 		if (get_data(iodev, LM78_TEMP, interface, &byte))
 			{
-			sensor->value = (gfloat) byte;
+			sensor->raw_value = (gfloat) byte;
 			return TRUE;
 			}
 		}
@@ -307,7 +310,7 @@ get_freebsd_sensor(Sensor *sensor)
 			{
 			if (byte == 0)
 				return FALSE;
-			sensor->value = 1.35E6 / (gfloat) byte;
+			sensor->raw_value = 1.35E6 / (gfloat) byte;
 			return TRUE;
 			}
 		}
@@ -315,7 +318,7 @@ get_freebsd_sensor(Sensor *sensor)
 		{
 		if (get_data(iodev, LM78_VOLT(id), interface, &byte))
 			{
-			sensor->value = (gfloat) byte / 64.0;
+			sensor->raw_value = (gfloat) byte / 64.0;
 			return TRUE;
 			}
 		}
@@ -478,23 +481,25 @@ get_linux_sensor(Sensor *sensor)
 			if ((n = sscanf(buf, "%*d.%*d %*d.%*d %16s %16s %16s",
 						sbuf[0], sbuf[1], sbuf[2])) < 1)
 				return FALSE;
-			sensor->value = locale_float_fix(sbuf[n - 1]);
+			sensor->raw_value = locale_float_fix(sbuf[n - 1]);
 			}
 		else
-			sensor->value = t[n - 1];
+			sensor->raw_value = t[n - 1];
 		}
 	else if (sensor->type == SENSOR_FAN)
 		{
-		if (sscanf(buf, "%*f %f", &sensor->value) != 1)
+		if ((n = sscanf(buf, "%f %f", &t[0], &t[1])) > 0)
+			sensor->raw_value = t[n - 1];
+		else
 			return FALSE;
 		}
 	else if (sensor->type == SENSOR_VOLTAGE)
 		{
-		if (sscanf(buf, "%*f %*f %f", &sensor->value) != 1)
+		if (sscanf(buf, "%*f %*f %f", &sensor->raw_value) != 1)
 			{
 			if (sscanf(buf, "%*d.%*d %*d.%*d %16s", sbuf[0]) != 1)
 				return FALSE;
-			sensor->value = locale_float_fix(sbuf[0]);
+			sensor->raw_value = locale_float_fix(sbuf[0]);
 			}
 		}
 	else
@@ -781,15 +786,15 @@ get_netbsd_sensor(Sensor *sensor)
 
    switch(data.units) {
       case ENVSYS_STEMP:	/* values in uK */
-	 sensor->value = (data.cur.data_us / 1.0e6) - 273.15/*0K*/ ;
+	 sensor->raw_value = (data.cur.data_us / 1.0e6) - 273.15/*0K*/ ;
 	 break;
 
       case ENVSYS_SFANRPM:	/* values in RPM */
-	 sensor->value = data.cur.data_us;
+	 sensor->raw_value = data.cur.data_us;
 	 break;
 
       default:			/* values in uV */
-	 sensor->value = data.cur.data_s / 1.0e6;
+	 sensor->raw_value = data.cur.data_s / 1.0e6;
 	 break;
    }
 
@@ -836,8 +841,42 @@ setup_sensor_interface(void)
 	}
 
 /* ======================================================================== */
+static gboolean	use_threads,
+				units_fahrenheit;
 
-static gint		units_fahrenheit;
+#if defined(GKRELLM_THREADS)
+static gboolean thread_busy;
+
+static void
+read_sensors_thread(void *data)
+	{
+	GList	*list;
+	Sensor	*sensor;
+
+	for (list = sensor_list; list; list = list->next)
+		{
+		sensor = (Sensor *) list->data;
+		if (sensor->type == SENSOR_TEMPERATURE && get_temperature)
+			(*get_temperature)(sensor);
+		if (sensor->type == SENSOR_FAN && get_fan)
+			(*get_fan)(sensor);
+		if (sensor->type == SENSOR_VOLTAGE && get_voltage)
+			(*get_voltage)(sensor);
+		}
+	thread_busy = FALSE;
+	}
+
+static void
+run_sensors_thread(void)
+	{
+	gpointer	thread_id;
+
+	if (thread_busy)
+		return;
+	thread_busy = TRUE;
+	thread_id = new_thread(read_sensors_thread, NULL);
+	}
+#endif	/* GKRELLM_THREADS */
 
 
   /* Sort so that sensors are ordered: temp, fan, voltage.
@@ -1103,14 +1142,14 @@ gkrellm_sensor_read_temperature(gchar *name, gfloat *temp, gchar *units)
 	gint	found_temp = FALSE;
 
 	if (   (sensor = get_mapped_sensor(name, SENSOR_TEMPERATURE)) != NULL
-		&& get_temperature && (*get_temperature)(sensor)
+		&& get_temperature
 	   )
 		{
-		t = sensor->value * sensor->factor + sensor->offset;
+		found_temp = use_threads ? TRUE : (*get_temperature)(sensor);
+		sensor->value = sensor->raw_value * sensor->factor + sensor->offset;
 		if (units_fahrenheit)
-			t = 1.8 * t + 32.0;
-		sensor->value = t;
-		found_temp = TRUE;
+			sensor->value = 1.8 * sensor->value + 32.0;
+		t = sensor->value;
 		}
 	if (! found_temp && GK.demo)
 		{
@@ -1122,9 +1161,9 @@ gkrellm_sensor_read_temperature(gchar *name, gfloat *temp, gchar *units)
 	if (units)
 		*units = units_fahrenheit ? 'F':'C';
 	if (GK.debug_level & DEBUG_SENSORS)
-		printf("read_temperature: %s(%d) t=%f\n", name, found_temp, t);
+		printf("read_temp: %s(%d) t=%f\n", name, found_temp, sensor->value);
 	if (found_temp && sensor)
-		gkrellm_check_alert(sensor->alert, t);
+		gkrellm_check_alert(sensor->alert, sensor->value);
 	return found_temp;
 	}
 
@@ -1135,16 +1174,16 @@ gboolean
 gkrellm_sensor_read_fan(gchar *name, gfloat *fan)
 	{
 	Sensor	*sensor;
-	gfloat	f	= 0;
+	gfloat	f = 0;
 	gint	found_fan = FALSE;
 
 	if (   (sensor = get_mapped_sensor(name, SENSOR_FAN)) != NULL
-		&& get_fan && (*get_fan)(sensor)
+		&& get_fan
 	   )
 		{
-		sensor->value *= sensor->factor;
+		found_fan = use_threads ? TRUE : (*get_fan)(sensor);
+		sensor->value = sensor->raw_value * sensor->factor;
 		f = sensor->value;
-		found_fan = TRUE;
 		}
 	if (! found_fan && GK.demo)
 		{
@@ -1154,27 +1193,27 @@ gkrellm_sensor_read_fan(gchar *name, gfloat *fan)
 	if (fan)
 		*fan = f;
 	if (GK.debug_level & DEBUG_SENSORS)
-		printf("read_fan: %s(%d) rpm=%f\n", name, found_fan, f);
+		printf("read_fan: %s(%d) rpm=%f\n", name, found_fan, sensor->value);
 	if (found_fan && sensor)
-		gkrellm_check_alert(sensor->alert, f);
+		gkrellm_check_alert(sensor->alert, sensor->value);
 	return found_fan;
 	}
 
 static gboolean
 sensor_read_voltage(Sensor *sensor, gfloat *voltage)
 	{
-	gfloat	v = 0;
-	gfloat	offset;
-	gint	found_voltage = FALSE;
+	gfloat		v = 0;
+	gfloat		offset;
+	gboolean	found_voltage = FALSE;
 
-	if (sensor && get_voltage && (*get_voltage)(sensor))
+	if (sensor && get_voltage)
 		{
+		found_voltage = use_threads ? TRUE : (*get_voltage)(sensor);
 		offset = sensor->offset;
 		if (sensor->vref)	/* A negative voltage is level shifted by vref */
 			offset *= sensor->vref->value;
-		sensor->value = sensor->value * sensor->factor + offset;
+		sensor->value = sensor->raw_value * sensor->factor + offset;
 		v = sensor->value;
-		found_voltage = TRUE;
 		}
 	if (! found_voltage && GK.demo)
 		{
@@ -1184,9 +1223,10 @@ sensor_read_voltage(Sensor *sensor, gfloat *voltage)
 	if (voltage)
 		*voltage = v;
 	if (GK.debug_level & DEBUG_SENSORS)
-		printf("read_voltage: %s(%d) v=%f\n", sensor->name, found_voltage, v);
+		printf("read_voltage: %s(%d) v=%f\n", sensor->name, found_voltage,
+				sensor->value);
 	if (found_voltage && sensor)
-		gkrellm_check_alert(sensor->alert, v);
+		gkrellm_check_alert(sensor->alert, sensor->value);
 	return found_voltage;
 	}
 
@@ -1367,10 +1407,13 @@ any_negative_volts(void)
 	Sensor		*s;
 	VoltMon		*volt;
 	gfloat		v;
+	gboolean	tmp, result = FALSE;
 
 	/* This routine can be called before any volt decals exist, but reading
 	|  voltages can trigger alerts which expect to find decals.  Hence freeze.
 	*/
+	tmp = use_threads;
+	use_threads = FALSE;	/* Need results immediately */
 	for (list = volt_list; list; list = list->next)
 		{
 		volt = (VoltMon *) list->data;
@@ -1381,9 +1424,13 @@ any_negative_volts(void)
 		sensor_read_voltage(volt->sensor, &v);
 		gkrellm_thaw_alert(volt->sensor->alert);
 		if (v < 0.0)
-			return TRUE;
+			{
+			result = TRUE;
+			break;
+			}
 		}
-	return FALSE;
+	use_threads = tmp;
+	return result;
 	}
 
 static VoltMon *
@@ -1570,10 +1617,13 @@ draw_voltages(gint do_names)
 static void
 update_voltage(void)
 	{
-	if (GK.five_second_tick)
-		{
-		draw_voltages(FALSE);
-		}
+	if (!GK.five_second_tick)
+		return;
+#if defined(GKRELLM_THREADS)
+	if (use_threads)
+		run_sensors_thread();
+#endif
+	draw_voltages(FALSE);
 	}
 
 static gint
@@ -1792,6 +1842,9 @@ save_sensors_config(FILE *f)
 		}
 	fprintf(f, "%s units_fahrenheit %d\n", SENSOR_CONFIG_KEYWORD,
 				units_fahrenheit);
+#if defined(GKRELLM_THREADS)
+	fprintf(f, "%s use_threads %d\n", SENSOR_CONFIG_KEYWORD, use_threads);
+#endif
 	fprintf(f, "%s volt_display_mode %d\n", SENSOR_CONFIG_KEYWORD,
 				display_mode);
 	}
@@ -1815,13 +1868,17 @@ load_sensors_config(gchar *arg)
 		printf("load_sensors_config: <%s> <%s>\n", config, item);
 	if (!strcmp(config, "units_fahrenheit"))
 		sscanf(item, "%d", &units_fahrenheit);
+#if defined(GKRELLM_THREADS)
+	else if (!strcmp(config, "use_threads"))
+		sscanf(item, "%d", &use_threads);
+#endif
 	else if (!strcmp(config, "volt_display_mode"))
 		sscanf(item, "%d", &display_mode);
 	else if (!strcmp(config, "vref"))
 		{
 		sscanf(item, "%63s %63s", name, item1);
-		s = lookup_sensor_from_basename(name);
-		s->vref = lookup_sensor_from_basename(item1);
+		if ((s = lookup_sensor_from_basename(name)) != NULL)
+			s->vref = lookup_sensor_from_basename(item1);
 		}
 	else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
 		{
@@ -1854,6 +1911,10 @@ load_sensors_config(gchar *arg)
 /* ------------------------------------------------------------------- */
 static GtkWidget	*fahrenheit_button;
 
+#if defined(GKRELLM_THREADS)
+static GtkWidget	*use_threads_button;
+#endif
+
 static GtkWidget	*display_mode_button[2];
 
 static void
@@ -1903,10 +1964,13 @@ apply_sensors_config(void)
 	GList	*list;
 	Sensor	*s;
 	gchar	*name;
-	gint	i, previous;
+	gint	i, previous, tmp;
 
 	previous = units_fahrenheit;
 	units_fahrenheit = GTK_TOGGLE_BUTTON(fahrenheit_button)->active;
+#if defined(GKRELLM_THREADS)
+	use_threads = GTK_TOGGLE_BUTTON(use_threads_button)->active;
+#endif
 	for (list = sensor_list; list; list = list->next)
 		{
 		s = (Sensor *) list->data;
@@ -1939,10 +2003,12 @@ apply_sensors_config(void)
 			}
 		if (s->vref_entry)
 			{
+			tmp = voltages_modified;
 			name = gkrellm_entry_get_text(&s->vref_entry);
 			s->vref = lookup_vref(s, name);
 			gtk_entry_set_text(GTK_ENTRY(s->vref_entry),
 						s->vref ? g_basename(s->vref->path) : "");
+			voltages_modified = tmp;
 			}
 		if (s->factor_spin_button)
 			s->factor = gtk_spin_button_get_value_as_float(
@@ -2231,7 +2297,11 @@ create_sensors_tab(GtkWidget *tab_vbox)
 		set_alert_button_state(sr);
 		}
 	gkrellm_check_button(vbox, &fahrenheit_button, units_fahrenheit, FALSE, 2,
-				_("Display fahrenheit"));
+		_("Display fahrenheit"));
+#if defined(GKRELLM_THREADS)
+	gkrellm_check_button(vbox, &use_threads_button, use_threads, FALSE, 2,
+		_("Read sensors in a thread (for slow sensor interfaces)"));
+#endif
 
 /* --Voltages tab */
 	vbox = gkrellm_create_framed_tab(tabs, _("Voltages"));
